@@ -16,11 +16,11 @@ cpop.class<-function(y,x,beta,sd,changepoints)
 
 
 
-#' A function for calculating the cost of a model fitted by cpop
+#' Calculate the cost of a model fitted by cpop
 #'
 #' @name cost
 #'
-#' @description Calculates the penalised cost of a model fitted by cpop using the residual sum of squares and the penalty values.
+#' @description Calculate the penalised cost of a model fitted by cpop using the residual sum of squares and the penalty values.
 #'
 #' @param object An instance of an S4 class produced by \code{\link{cpop}}.
 #'
@@ -46,7 +46,7 @@ cpop.class<-function(y,x,beta,sd,changepoints)
 #' @aliases cost,cpop.class-method
 #'
 #' @export
-setGeneric("cost",function(object) {standardGeneric("cost")})
+if(!isGeneric("cost")) {setGeneric("cost",function(object) {standardGeneric("cost")})}
 setMethod("cost",signature=list("cpop.class"),
           function(object)
           {
@@ -55,7 +55,7 @@ setMethod("cost",signature=list("cpop.class"),
           })	      
 
 
-#' A function for generating simulated data
+#' Generate simulated data
 #'
 #' @name simchangeslope
 #'
@@ -101,18 +101,18 @@ setMethod("cost",signature=list("cpop.class"),
 #' @export
 simchangeslope<-function(x,changepoints,change.slope,sd=1)
 {
-  K=length(changepoints)
-  mu=rep(0,length(x))
+  K <- length(changepoints)
+  mu <- rep(0,length(x))
   for(k in 1:K)
   {
-     mu=mu+change.slope[k]*pmax(x-changepoints[k],0)
+     mu <- mu+change.slope[k]*pmax(x-changepoints[k],0)
   }
-  y=mu+rnorm(length(x),0.0,sd)
+  y <- mu+rnorm(length(x),0.0,sd)
   return(y)
 }
 
 
-#' Visualisation of Changepoint Locations and Data
+#' Visualise changepoint locations and associated data
 #'
 #' @name plot
 #'
@@ -175,7 +175,7 @@ setMethod("plot",signature=list("cpop.class"),function(x)
 
 
 
-#' Summary of cpop Analysis.
+#' Summarise a cpop analysis
 #'
 #' @name summary
 #'
@@ -240,7 +240,7 @@ setMethod("summary",signature=list("cpop.class"),function(object)
 })
 
 
-#' Displays an S4 object produced by cpop.
+#' Display the S4 object produced by cpop
 #'
 #' @name show
 #'
@@ -269,7 +269,7 @@ setMethod("summary",signature=list("cpop.class"),function(object)
 #' show(res)
 #'
 #' @export
-setGeneric("show",function(object) {standardGeneric("show")})
+if(!isGeneric("show")) {setGeneric("show",function(object) {standardGeneric("show")})}
 setMethod("show",signature=list("cpop.class"),function(object)
 {
     summary(object)
@@ -277,7 +277,7 @@ setMethod("show",signature=list("cpop.class"),function(object)
 })
 
 
-#' Extract Model Fitted Values
+#' Extract model fitted values
 #'
 #' @name fitted
 #'
@@ -310,12 +310,12 @@ setMethod("show",signature=list("cpop.class"),function(object)
 #' fitted(res)
 
 #' @export
-setGeneric("fitted",function(object) {standardGeneric("fitted")})
+# if(!isGeneric("fitted")) {setGeneric("fitted",function(object) {standardGeneric("fitted")})}
 setMethod("fitted",signature=list("cpop.class"),
           function(object)
           {
 	  x<-sort(unique(c(object@x,object@changepoints)))
-	  y_hat<-estimate(object,x)$y_hat
+	  y_hat<-estimate.util(object,x)$y_hat
 	  cpts<-object@changepoints
 	  y_0<-unlist(Map(function(val) y_hat[which(x==val)],cpts))
   	  df<-data.frame("x0"=cpts[1:(length(cpts)-1)],
@@ -332,7 +332,7 @@ setMethod("fitted",signature=list("cpop.class"),
           })	      
 
 
-#' Changepoint Locations
+#' Changepoint locations
 #'
 #' @name changepoints
 #'
@@ -379,22 +379,26 @@ setMethod("changepoints",signature=list("cpop.class"),
 	      }
 	      return(df)
           })	      
-#' cpop
+
+#' Find the best segmentation of data for a change-in-slope model
+#'
+#' @name cpop
+#'
+#' @description The CPOP algorithm fits a change-in-slope model to data.
 #'
-#'  Algorithm for finding the best segmentation of data for a change-in-slope model.
-#' 
 #' @param y A vector of length n containing the data.
 #' @param x A vector of length n containing the locations of y. Default value is NULL, in which case the locations \code{x = 1:length(y)-1} are assumed.
 #' @param grid An ordered vector of possible locations for the change points. If this is NULL, then this is set to x, the vector of times/locations of the data points.
 #' @param beta A positive real value for the penalty incurred for adding a changepoint (prevents over-fitting). Default value is \code{2*log(length(y))}.
-#' @param sd Estimate of residual standard deviation. Can be a single numerical value or a vector of values for the case of varying standard deviation. Default value is 1.
+#' @param sd Estimate of residual standard deviation. Can be a single numerical value or a vector of values for the case of varying standard deviation.
+#' Default value is \code{sd = sqrt(mean(diff(diff(y))^2)/6)}.
 #' @param minseglen The minimum allowable segment length, i.e. distance between successive changepoints. Default is 0.
 #' @param prune.approx Only relevant if a minimum segment length is set. If True, cpop will use an approximate pruning algorithm that will speed up computation but may
 #' occasionally lead to a sub-optimal solution in terms of the estimate change point locations. If the minimum segment length is 0, then an exact pruning algorithm is possible and is used.
 #'
 #' @return An instance of an S4 class of type cpop.class.
 #'
-#' @description \loadmathjax{} The CPOP algorithm fits a change-in-slope model to data. It assumes that we have we have data points \mjeqn{(y_1,x_1),\ldots,(y_n,x_n)}{(y_1,x_1),...,(y_n,x_n)}, ordered
+#' @details \loadmathjax{} The CPOP algorithm fits a change-in-slope model to data. It assumes that we have we have data points \mjeqn{(y_1,x_1),\ldots,(y_n,x_n)}{(y_1,x_1),...,(y_n,x_n)}, ordered
 #' so that \mjeqn{x_1 < x_2 < \cdots < x_n}{x_1 < x_2 < ... < x_n}. For example  \mjeqn{x_i}{x_i} could be a time-stamp of when response \mjeqn{y_i}{y_i} is obtained. We model the response, \mjeqn{y}{y}, as a signal
 #' plus noise where the signal is modelled as a continuous piecewise linear function of \mjeqn{x}{x}. That is \mjdeqn{y_i=f(x_i)+\epsilon_i}{y_i=f(x_i)+e_i} where \mjeqn{f(x)}{f(x)} is a continuous
 #' piecewise linear function.
@@ -420,6 +424,8 @@ setMethod("changepoints",signature=list("cpop.class"),
 #' and \mjeqn{\beta}{beta}: e.g. if we double the variances and half the value of \mjeqn{\beta}{beta} we will obtain the same estimates for the number and location of the changes and the
 #' underlying function.
 #'
+#' @rdname cpop
+#'
 #' @references \insertRef{doi:10.1080/10618600.2018.1512868}{cpop}
 #'
 #' @examples
@@ -462,11 +468,28 @@ setMethod("changepoints",signature=list("cpop.class"),
 #' print(p)
 #'
 #' @export
-cpop<-function(y,x=1:length(y)-1,grid=x,beta=2*log(length(y)),sd=1,minseglen=0,prune.approx=FALSE)
+cpop<-function(y,x=1:length(y)-1,grid=x,beta=2*log(length(y)),sd=sqrt(mean(diff(diff(y))^2)/6),minseglen=0,prune.approx=FALSE)
 {
+    if(base::missing(beta))
+    {
+       cat("No value set for beta, so the default value of beta=2log(n), where n is the number of data points, has been used.","\n",
+           "This default value is appropriate if the noise is IID Gaussian and the value of sd is a good estimate of the standard","\n",
+	   "deviation of the noise. If these assumptions do not hold, the estimate of the number of changepoints may be inaccurate.","\n",
+	   "To check robustness use cpop.crops with beta_min and beta_max arguments.","\n\n",sep="")
+    }
+    if(base::missing(sd))
+    {
+       cat("No value set for sd. An estimate for the noise standard deviation based on the variance of the second differences of","\n",
+           "the data has been used. If this estimate is too small it may lead to over-estimation of changepoints. You are advised","\n",
+	   "to check this by comparing the standard deviation of the residuals to the estimated value used for sd.","\n",sep="")
+    }
+    if(is.null(sd))
+    {
+      sd <- sqrt(mean(diff(diff(y))^2)/6)
+    }
     if(length(sd)!=length(y))
     {
-      sd=rep(sd[1],length(y))
+      sd <- rep(sd[1],length(y))
     }
     sigsquared<-sd^2
     if(minseglen != 0)
@@ -483,17 +506,17 @@ cpop<-function(y,x=1:length(y)-1,grid=x,beta=2*log(length(y)),sd=1,minseglen=0,p
 
 design<-function(object,x=object@x)
 {
-  n=length(x)
+  n <- length(x)
   cpts<-object@changepoints[-1]
-  p=length(cpts)
-  X=matrix(NA,nrow=n,ncol=p+1)
-  X[,1]=1
-  X[,2]=x-x[1]
+  p <- length(cpts)
+  X <- matrix(NA,nrow=n,ncol=p+1)
+  X[,1] <- 1
+  X[,2] <- x-x[1]
   if(p>1)
   {
     for(i in 1:(p-1))
     {
-      X[,i+2]=pmax(rep(0,n),x-cpts[i])
+      X[,i+2] <- pmax(rep(0,n),x-cpts[i])
     }
   }
   return(X)
@@ -501,7 +524,7 @@ design<-function(object,x=object@x)
 
 parameters<-function(object)
 {
-  n=length(object@y)
+  n <- length(object@y)
   cpts<-object@changepoints[-1]	
   p<-length(cpts)
   W<-diag(object@sd^-2)
@@ -511,7 +534,13 @@ parameters<-function(object)
   return(pars)
 }
 
-#' A function for estimating the fit of a cpop model
+estimate.util <- function(object,x)
+{
+    return(data.frame("x"=x,"y_hat"=design(object,x)%*%parameters(object)))
+}
+
+
+#' Estimate the fit of a cpop model
 #'
 #' @name estimate
 #'
@@ -549,15 +578,28 @@ parameters<-function(object)
 #' estimate(res,seq(-20,140,20))
 #' 
 #' @export
-setGeneric("estimate",function(object,x=object@x,...) {standardGeneric("estimate")})
+if(!isGeneric("estimate")) {setGeneric("estimate",function(object,x=object@x,...) {standardGeneric("estimate")})}
 setMethod("estimate",signature=list("cpop.class"),
           function(object,x)
           {
-             return(data.frame("x"=x,"y_hat"=design(object,x)%*%parameters(object)))
+  	        df<-fitted(object)
+        	nrows <- nrow(df)
+        	df <- rbind(df[1,],df,df[nrows,])
+        	df$x1[1] <- df$x0[1]
+        	df$x0[1] <- -Inf
+        	df$x0[nrows+2] <- df$x1[nrows+2]
+        	df$x1[nrows+2] <- Inf
+        	y_hat <- Map(function(val)
+                     {    
+                        interval <- df[val > df$x0 & val <= df$x1,]
+                        return(val*interval$gradient+interval$intercept)
+                     },
+                     x)
+        	return(data.frame("x"=x,"y_hat"=unlist(y_hat)))
           })	      
 
 
-#' Extracts residuals from a cpop model
+#' Extract residuals from a cpop model
 #'
 #' @name residuals
 #'
@@ -601,28 +643,28 @@ setMethod("residuals",signature=list("cpop.class"),
 
 cpop.fit<-function(y,x,out.changepoints,sigsquared)
 {
-  n=length(y)
-  if(length(sigsquared)!=n)
+  n <- length(y)
+  if(length(sigsquared) != n)
   {
-     sigsquared=rep(sigsquared[1],n)
+     sigsquared <- rep(sigsquared[1],n)
   }
-  p=length(out.changepoints)
-  W=diag(sigsquared^-1)
-  X=matrix(NA,nrow=n,ncol=p+1)
-  X[,1]=1
-  X[,2]=x-x[1]
+  p <- length(out.changepoints)
+  W <- diag(sigsquared^-1)
+  X <- matrix(NA,nrow=n,ncol=p+1)
+  X[,1] <- 1
+  X[,2] <- x-x[1]
   if(p>1)
   {
     for(i in 1:(p-1))
     {
-      X[,i+2]=pmax(rep(0,n),x-out.changepoints[i])
+      X[,i+2] <- pmax(rep(0,n),x-out.changepoints[i])
 
     }
   }
-  XTX=t(X)%*%W%*%X
-  beta=as.vector(solve(XTX)%*%t(X)%*%W%*%y)
-  fit=X%*%beta
-  residuals=y-fit
+  XTX <- t(X)%*%W%*%X
+  beta <- as.vector(solve(XTX)%*%t(X)%*%W%*%y)
+  fit <- X%*%beta
+  residuals <- y-fit
   return(list(fit=fit,residuals=residuals,X=X,pars=beta))
 }
 
@@ -661,7 +703,7 @@ CPOP.uneven_impl<-function(y,x,beta,sigsquared=1)
     SP[i+1]<-SP[i]+1/sigsquared[i]
   }
   
-  x0=c(2*x[1]-x[2],x)
+  x0 <- c(2*x[1]-x[2],x)
   
   coeffs<-matrix(0,ncol=5,nrow=1) #first two columns are current time point and most recent changepoint, final three are coefficients for cost
   coeffs[1,5]<--beta
@@ -673,16 +715,16 @@ CPOP.uneven_impl<-function(y,x,beta,sigsquared=1)
   ## code minimise 1/(2sd^2)*RSS rather than RSS/sd^2
   ## hence need to have sd^2 
  #### 
- sigsquared=sigsquared/2
+ sigsquared <- sigsquared/2
 
   for(taustar in 1:n){
     new.CPvec<-paste(CPvec,taustar,sep=",")
     ##update coefficients --THIS HAS BEEN CHANGED FROM CPOP CODE
     ##CURRENTLY CHANGE ONLY IN R CODE VERSION
    #if(useC==FALSE){
-     new.coeffs=coeff.update.uneven.var(coeffs,S,SXY,SS,SX,SX2,SP,x0,taustar,beta)
+     new.coeffs <- coeff.update.uneven.var(coeffs,S,SXY,SS,SX,SX2,SP,x0,taustar,beta)
     #}    else if(useC==TRUE){
-    #  new.coeffs=coeff.update.c(coeffs,S,SJ,SS,taustar,sigsquared,beta)
+    #  new.coeffs <- coeff.update.c(coeffs,S,SJ,SS,taustar,sigsquared,beta)
     #} else{stop("useC must be a TRUE or FALSE value")
     #}
    # if(taustar==2){browser()}
@@ -690,22 +732,22 @@ CPOP.uneven_impl<-function(y,x,beta,sigsquared=1)
     ###################################################pruning bit##########  
     if(length(new.coeffs[,1])>1){
       ##added###
-      #keep1=prune1(new.coeffs,taustar) ##first pruning
-      keep1=1:length(new.coeffs[,1])
-      new.coeffs.p=new.coeffs[keep1,]
-      new.CPvec=new.CPvec[keep1]
+      #keep1 <- prune1(new.coeffs,taustar) ##first pruning
+      keep1 <- 1:length(new.coeffs[,1])
+      new.coeffs.p <- new.coeffs[keep1,]
+      new.CPvec <- new.CPvec[keep1]
       ###########
       if(sum(keep1)>1){
-       keep2=prune2.c(new.coeffs.p)
-       new.coeffs.p=new.coeffs.p[keep2,]
-       new.CPvec=new.CPvec[keep2]
+       keep2 <- prune2.c(new.coeffs.p)
+       new.coeffs.p <- new.coeffs.p[keep2,]
+       new.CPvec <- new.CPvec[keep2]
       }
     }else{
-      new.coeffs.p=new.coeffs
+      new.coeffs.p <- new.coeffs
     }
     ####PELT PRUNE############################
     if(taustar>2){
-      keeppelt=peltprune(new.coeffs,beta)
+      keeppelt <- peltprune(new.coeffs,beta)
       coeffs<-coeffs[keeppelt,]
       CPvec<-CPvec[keeppelt]
     }
@@ -740,18 +782,18 @@ CPOP.uneven_impl<-function(y,x,beta,sigsquared=1)
 ###avoids loop
 ########################################################################################
 
-coeff.update.uneven.var=function(coeffs,S,SXY,SS,SX,SX2,SP,x0,taustar,beta){
+coeff.update.uneven.var <- function(coeffs,S,SXY,SS,SX,SX2,SP,x0,taustar,beta){
   
   coeff.new<-coeffs
-  coeff.new[,2]=coeffs[,1] 
+  coeff.new[,2] <- coeffs[,1] 
   coeff.new[,1]<-taustar
   
   sstar<-coeff.new[,2]
   Xs<-x0[sstar+1]
   Xt<-x0[taustar+1]
-  seglen=Xt-Xs
+  seglen <- Xt-Xs
   
-  n.obs=taustar-sstar
+  n.obs <- taustar-sstar
   #A<-(SX2[taustar+1]-SX2[sstar+1]-2*Xs*(SX[taustar+1]-SX[sstar+1])+(SP[taustar+1]-SP[sstar+1])*Xs^2)/(seglen^2)
   #B<- 2*( (Xt+Xs)*(SX[taustar+1]-SX[sstar+1])-(SP[taustar+1]-SP[sstar+1])*Xt*Xs-(SX2[taustar+1]-SX2[sstar+1]))/(seglen^2)
   #C<-(-2)/(seglen)*(SXY[taustar+1]-SXY[sstar+1]-Xs*(S[taustar+1]-S[sstar+1]))
@@ -768,10 +810,10 @@ coeff.update.uneven.var=function(coeffs,S,SXY,SS,SX,SX2,SP,x0,taustar,beta){
   FF<-coeffs.cpp[[6]]
 
 
-  m=length(sstar)
-  ind1=(1:m)[FF==0 & coeffs[,3]==0 & B==0]
-  ind2=(1:m)[FF==0 & coeffs[,3]==0 & B!=0]
-  ind3=(1:m)[!(FF==0 & coeffs[,3]==0)]
+  m <- length(sstar)
+  ind1 <- (1:m)[FF==0 & coeffs[,3]==0 & B==0]
+  ind2 <- (1:m)[FF==0 & coeffs[,3]==0 & B!=0]
+  ind3 <- (1:m)[!(FF==0 & coeffs[,3]==0)]
   if(length(ind1)>0){
     coeff.new[ind1,5]<-coeffs[ind1,5]+D[ind1]+beta
     coeff.new[ind1,4]<-C[ind1]
@@ -810,7 +852,7 @@ coeff.update.uneven.var=function(coeffs,S,SXY,SS,SX,SX2,SP,x0,taustar,beta){
 ###################################PELT pruning function###################################################
 ###########################################################################################################
 
-peltprune=function(x,beta){
+peltprune <- function(x,beta){
   minx<-x[,5]-x[,4]^2/(4*x[,3])
   return(which(minx<=(min(minx)+2*beta)))  
 }
