@@ -19,7 +19,8 @@ cpop.crops.class<-function(crops.result,y,x)
 #' @param grid An ordered vector of possible locations for the change points. If this is NULL, then this is set to x, the vector of times/locations of the data points.
 #' @param beta_min Minimum value of the penalty range to be searched. Default is \code{1.5*log(length(y))}
 #' @param beta_max Maximum value of the penalty range to be searched. Default is \code{2.5*log(length(y))} 
-#' @param sd Estimate of residual standard deviation. Can be a single numerical value or a vector of values for the case of varying standard deviation. Default value is 1.
+#' @param sd Estimate of residual standard deviation. Can be a single numerical value or a vector of values for the case of varying standard deviation.
+#' Default value is \code{sd = sqrt(mean(diff(diff(y))^2)/6)}.
 #' @param minseglen The minimum allowable segment length, i.e. distance between successive changepoints. Default is 0.
 #' @param prune.approx Only relevant if a minimum segment length is set. If TRUE, cpop will use an approximate pruning algorithm that will speed up computation but may
 #' occasionally lead to a sub-optimal solution in terms of the estimate change point locations. If the minimum segment length is 0, then an exact pruning algorithm is possible and is used. 
@@ -56,8 +57,15 @@ cpop.crops.class<-function(crops.result,y,x)
 #' @references \insertRef{crops-package}{cpop}
 #' @export
 cpop.crops<-function(y,x = 1:length(y),grid = x, beta_min = 1.5 * log(length(y)),
-                     beta_max = 2.5 * log(length(y)),sd = 1,minseglen = 0,prune.approx = FALSE)
+                     beta_max = 2.5 * log(length(y)),sd = sqrt(mean(diff(diff(y))^2)/6),
+		     minseglen = 0,prune.approx = FALSE)
 {
+    if(base::missing(sd))
+    {
+       cat("No value set for sd. An estimate for the noise standard deviation based on the variance of the second differences of","\n",
+           "the data has been used. If this estimate is too small it may lead to over-estimation of changepoints. You are advised","\n",
+	   "to check this by comparing the standard deviation of the residuals to the estimated value used for sd.","\n",sep="")
+    }
     CPT.init <- function(y,x,grid,sd,minseglen,prune.approx)
     {
        return(
@@ -80,7 +88,7 @@ cpop.crops<-function(y,x = 1:length(y),grid = x, beta_min = 1.5 * log(length(y))
 }
 
 
-#' Obtain the cpop models created by cpop.crops
+#' Extract the cpop models created by cpop.crops
 #'
 #' @name cpop.crops.models
 #'
